#include <Arduino.h>
#include <io/button.h>

using io::IO;
using io::button::Button;
using io::button::Event;

const int LED_PIN = LED_BUILTIN;
int LED_VALUE = 0;

auto pin1 = IO.digitalInput(1);
Button powerButton(&pin1);

void toggleLed() {
  LED_VALUE = (LED_VALUE + 1) % 2;
  digitalWrite(LED_PIN, LED_VALUE);
}

auto helloListener = Button::createListener([](const Event& event){
  Serial.println("On Press");
  Serial.println(&event.source == &powerButton);
}, [](const Event& event){
  Serial.println("On Release");
  Serial.println(&event.source == &powerButton);
}, [](const Event& event){
  Serial.println("On Hold");
  Serial.println(&event.source == &powerButton);
});

void setup() {
  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, LED_VALUE);
  powerButton.setListener(&helloListener);
}


unsigned long lastNow = 0;

void loop() {
  unsigned long now = millis();
  unsigned long delta = now - lastNow;
  lastNow = now;
  powerButton.tick(delta);
  delay(powerButton.getDebounceDelay());
}
