#include "io/button.h"

namespace io {
namespace button {

using io::Input;
using io::IO;

void noop(const Event& event) {}

Button::Listener NULL_LISTENER;

void Button::Listener::onPress(const Event& event) {}

void Button::Listener::onRelease(const Event& event) {}

void Button::Listener::onHold(const Event& event) {}

Button::Button(Input<bool>* input, bool reversed) : input(input), reversed(reversed), lastValue(reversed), listener(&NULL_LISTENER) {}

Button::~Button() {}

void Button::setListener(Listener* listener) { this->listener = listener; }

void Button::unsetListener() { this->listener = &NULL_LISTENER; }

bool Button::read() const { return this->input->read() ^ this->reversed; }

void Button::fireOnPress(unsigned long delta) {
  this->state = State::PRESSED;
  if (this->lastPressTimer <= this->multipleClickThreshold) {
    this->clickCounter += 1;
  }
  else {
    this->clickCounter = 1;
  }
  this->lastPressTimer = 0;
  this->listener->onPress(Event{
      .source = *this,
      .state = this->state,
      .delta = delta,
      .timer = this->lastReleaseTimer,
      .clicksCounter =this->clickCounter,
  });
}

void Button::fireOnRelease(unsigned long delta) {
  this->state = State::RELEASED;
  this->lastReleaseTimer = 0;
  this->listener->onRelease(Event{
      .source = *this,
      .state = this->state,
      .delta = delta,
      .timer = this->lastPressTimer,
      .clicksCounter = this->clickCounter,
  });
}

void Button::fireOnHold(unsigned long delta) {
  this->listener->onHold(Event{
      .source = *this,
      .state = this->state,
      .delta = delta,
      .timer = this->lastPressTimer,
      .clicksCounter = this->clickCounter,
  });
}

unsigned int Button::getDebounceDelay() const { return this->debounceDelay; }

void Button::setDebounceDelay(unsigned int debounceDelay) { this->debounceDelay = debounceDelay; }

unsigned int Button::getMultipleClickThreshold() const { return this->multipleClickThreshold; }

void Button::setMultipleClickThreshold(unsigned int multipleClickThreshold) { this->multipleClickThreshold = multipleClickThreshold; }

void Button::tick(unsigned long delta) {
  this->lastPressTimer += delta;
  this->lastReleaseTimer += delta;
  bool value = this->read();
  bool changed = !this->lastValue == value;
  if (changed) {
    this->debounceTimer = 0;
    this->lastValue = value;
  }
  this->debounceTimer += delta;
  if (this->debounceTimer < this->debounceDelay) {
    return;
  }
  if (value) {
    if (this->state == State::RELEASED) {
      this->fireOnPress(delta);
    }
    else {
      this->fireOnHold(delta);
    }
  }
  else if (this->state == State::PRESSED) {
    this->fireOnRelease(delta);
  }
}

} // namespace button

} // namespace io
