#include "ArduinoIO.h"
#include <Arduino.h>

#ifdef USE_ARDUINO_IO

namespace io {

ArduinoAnalogOutput::ArduinoAnalogOutput(unsigned char pinNumber) {
  this->pinNumber = pinNumber;
  pinMode(this->pinNumber, OUTPUT);
}

ArduinoAnalogOutput::~ArduinoAnalogOutput() { pinMode(this->pinNumber, INPUT); }

void ArduinoAnalogOutput::write(double value) { analogWrite(this->pinNumber, value * 255); }

ArduinoDigitalOutput::ArduinoDigitalOutput(unsigned char pinNumber) {
  this->pinNumber = pinNumber;
  pinMode(this->pinNumber, OUTPUT);
}

ArduinoDigitalOutput::~ArduinoDigitalOutput() { pinMode(this->pinNumber, INPUT); }

void ArduinoDigitalOutput::write(bool value) { digitalWrite(this->pinNumber, value ? HIGH : LOW); }

ArduinoDigitalInput::ArduinoDigitalInput(unsigned char pinNumber, unsigned char mode) {
  this->pinNumber = pinNumber;
  pinMode(this->pinNumber, mode);
}

ArduinoDigitalInput::~ArduinoDigitalInput() { pinMode(this->pinNumber, INPUT); }

bool ArduinoDigitalInput::read() const { return digitalRead(this->pinNumber) > 0; }

ArduinoAnalogInput::ArduinoAnalogInput(unsigned char pinNumber, unsigned char mode) {
  this->pinNumber = pinNumber;
  pinMode(this->pinNumber, mode);
}

ArduinoAnalogInput::~ArduinoAnalogInput() { pinMode(this->pinNumber, INPUT); }

double ArduinoAnalogInput::read() const { return analogRead(this->pinNumber) > 0; }


ArduinoIO::ArduinoIO() {}

ArduinoAnalogOutput ArduinoIO::analogOutput(unsigned char pinNumber) { return io::ArduinoAnalogOutput(pinNumber); }

ArduinoDigitalOutput ArduinoIO::digitalOutput(unsigned char pinNumber) { return io::ArduinoDigitalOutput(pinNumber); }

ArduinoAnalogInput ArduinoIO::analogInput(unsigned char pinNumber) { return io::ArduinoAnalogInput(pinNumber, INPUT); }

ArduinoDigitalInput ArduinoIO::digitalInput(unsigned char pinNumber) { return io::ArduinoDigitalInput(pinNumber, INPUT); }

ArduinoDigitalInput ArduinoIO::digitalInput(unsigned char pinNumber, unsigned char mode) { return io::ArduinoDigitalInput(pinNumber, mode); }

ArduinoDigitalInput ArduinoIO::digitalInputWithPullup(unsigned char pinNumber) { return io::ArduinoDigitalInput(pinNumber, INPUT_PULLUP); }

ArduinoIO IO;

} // namespace io



#endif