#ifndef __IO__ARDUINOIO__
#define __IO__ARDUINOIO__

#include "../IO.h"

namespace io {

class ArduinoAnalogOutput : public Output<double> {
protected:
  unsigned char pinNumber;

public:
  ArduinoAnalogOutput(unsigned char pinNumber);
  virtual ~ArduinoAnalogOutput();
  void write(double value);
};

class ArduinoDigitalOutput : public Output<bool> {
protected:
  unsigned char pinNumber;

public:
  ArduinoDigitalOutput(unsigned char pinNumber);
  virtual ~ArduinoDigitalOutput();
  void write(bool value);
};

class ArduinoDigitalInput : public Input<bool> {
protected:
  unsigned char pinNumber;

public:
  ArduinoDigitalInput(unsigned char pinNumber, unsigned char pinMode);
  virtual ~ArduinoDigitalInput();
  bool read() const;
};

class ArduinoAnalogInput : public Input<double> {
protected:
  unsigned char pinNumber;

public:
  ArduinoAnalogInput(unsigned char pinNumber, unsigned char pinMode);
  virtual ~ArduinoAnalogInput();
  double read() const;
};

class ArduinoIO : public BaseIO {
public:
  ArduinoIO();
  io::ArduinoAnalogOutput analogOutput(unsigned char pinNumber);
  io::ArduinoAnalogInput analogInput(unsigned char pinNumber);
  io::ArduinoDigitalOutput digitalOutput(unsigned char pinNumber);
  io::ArduinoDigitalInput digitalInput(unsigned char pinNumber);
  io::ArduinoDigitalInput digitalInput(unsigned char pinNumber, unsigned char pinMode);
  io::ArduinoDigitalInput digitalInputWithPullup(unsigned char pinNumber);
};


#ifdef USE_ARDUINO_IO
extern ArduinoIO IO;
#endif

} // namespace io

#endif