#include "ESP32IO.h"

#ifdef USE_ESP32_IO

namespace io {

ESP32DigitalOutput::ESP32DigitalOutput(unsigned char pinNumber) {
  this->pinNumber = pinNumber;
  pinMode(this->pinNumber, OUTPUT);
}

ESP32DigitalOutput::~ESP32DigitalOutput() { pinMode(this->pinNumber, INPUT); }

void ESP32DigitalOutput::write(bool value) { digitalWrite(this->pinNumber, value ? HIGH : LOW); }

ESP32DigitalInput::ESP32DigitalInput(unsigned char pinNumber, unsigned char mode) {
  this->pinNumber = pinNumber;
  pinMode(this->pinNumber, mode);
}

ESP32DigitalInput::~ESP32DigitalInput() { pinMode(this->pinNumber, INPUT); }

bool ESP32DigitalInput::read() const { return digitalRead(this->pinNumber) > 0; }

unsigned int ESP32AnalogOutput::calcDutyCycle(double value) {
  unsigned int maxValue = pow(2, this->bitResolution) - 1;
  double preparedValue = max(min(1., value), 0.);
  return round(preparedValue * maxValue);
}

ESP32AnalogOutput::ESP32AnalogOutput(unsigned char pinNumber) {
  this->channel = 0;
  this->bitResolution = 8;
  const unsigned int frequency = 5000;
  this->pinNumber = pinNumber;
  ledcAttachChannel(this->pinNumber, frequency, this->bitResolution, this->channel);
}

void ESP32AnalogOutput::write(double value) {
  unsigned int dutyCycle = this->calcDutyCycle(value);
  ledcWrite(this->pinNumber, dutyCycle);
}

ESP32IO::ESP32IO() {}

ESP32AnalogOutput ESP32IO::analogOutput(unsigned char pinNumber) { return ESP32AnalogOutput(pinNumber); }
ESP32DigitalOutput ESP32IO::digitalOutput(unsigned char pinNumber) { return ESP32DigitalOutput(pinNumber); }
ESP32DigitalInput ESP32IO::digitalInput(unsigned char pinNumber) { return ESP32DigitalInput(pinNumber, INPUT); }
ESP32DigitalInput ESP32IO::digitalInput(unsigned char pinNumber, unsigned char mode) { return ESP32DigitalInput(pinNumber, mode);};
ESP32DigitalInput ESP32IO::digitalInputWithPullup(unsigned char pinNumber) { return ESP32DigitalInput(pinNumber, INPUT_PULLUP);};

ESP32IO IO;

} // namespace io

#endif
