#ifndef __IO_ESP32IO__
#define __IO_ESP32IO__

#include <Arduino.h>

#include "../IO.h"

#ifdef USE_ESP32_IO

#define LED_BUILTIN 2

namespace io {

class ESP32DigitalOutput : public Output<bool> {
protected:
  unsigned char pinNumber;

public:
  ESP32DigitalOutput(unsigned char pinNumber);
  virtual ~ESP32DigitalOutput();
  void write(bool value);
};

class ESP32DigitalInput : public Input<bool> {
protected:
  unsigned char pinNumber;

public:
  ESP32DigitalInput(unsigned char pinNumber, unsigned char pinMode);
  virtual ~ESP32DigitalInput();
  bool read() const;
};

class ESP32AnalogOutput : public Output<double> {
protected:
  unsigned char channel;
  unsigned char bitResolution;
  unsigned char pinNumber;
  unsigned int calcDutyCycle(double value);

public:
  ESP32AnalogOutput(unsigned char pinNumber);
  void write(double value);
};


class ESP32IO : public BaseIO {
public:
  ESP32IO();
  ESP32AnalogOutput analogOutput(unsigned char pinNumber);
  ESP32DigitalOutput digitalOutput(unsigned char pinNumber);
  ESP32DigitalInput digitalInput(unsigned char pinNumber);
  ESP32DigitalInput digitalInput(unsigned char pinNumber, unsigned char mode);
  ESP32DigitalInput digitalInputWithPullup(unsigned char pinNumber);
};

extern ESP32IO IO;

} // namespace io


#endif
#endif